package cdc;

import bean.InputValue;
import com.alibaba.fastjson.JSON;
import com.ejlchina.okhttps.GsonMsgConvertor;
import com.ejlchina.okhttps.HTTP;
import org.apache.flink.api.common.functions.RichMapFunction;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.table.api.EnvironmentSettings;
import org.apache.flink.table.api.Table;
import org.apache.flink.table.api.bridge.java.StreamTableEnvironment;
import org.apache.flink.types.Row;
import java.util.HashMap;

/**
 * @program: sunwin_metro
 * @description: 根据mysql cdc 同步 mysql数据库中的表信息 构建neo4j图谱
 * @author: xuYJ
 * @create: 2021-06-23 09:20
 **/
public class NeoCdc {
    public static void main(String[] args) throws Exception {
        EnvironmentSettings build = EnvironmentSettings.newInstance()
                .useBlinkPlanner()
                .inStreamingMode()
                .build();

        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        env.setParallelism(1);
        StreamTableEnvironment tableEnv = StreamTableEnvironment.create(env, build);
        //源表用来同步mysql表
        String source =
                "CREATE TABLE failure_binlog (" +
                        "   phenomenon_name STRING ," +
                        "   operation_name STRING," +
                        "   reason_name STRING," +
                        "   reason_solve STRING," +
                        "   location_name STRING," +
                        "   location_abbreviation STRING," +
                        "   equipment_name STRING," +
                        "   equipment_abbreviation STRING" +
                        ") WITH (" +
                        "'connector' = 'mysql-cdc'," +
                        "'hostname' = '192.168.20.61'," +
                        "'port' = '3306'," +
                        "'username' = 'root'," +
                        "'password' = 'root'," +
                        "'database-name' = 'URTDB'," +
                        "'table-name' = 'metro_failure'" +
                        ")";
        tableEnv.executeSql(source);
        Table testSink = tableEnv.from("failure_binlog");
        //用 Row 对象接受mysql新增/更新/删除 数据
        DataStream<Tuple2<Boolean, Row>> stream = tableEnv.toRetractStream(testSink, Row.class);
        stream.map(new RichMapFunction<Tuple2<Boolean, Row>, String>() {
            HTTP http;
            long l;
            @Override
            public void open(Configuration parameters) throws Exception {
                super.open(parameters);
                http = HTTP.builder()
                        .baseUrl("http://localhost:8080")
                        .addMsgConvertor(new GsonMsgConvertor())
                        .build();
                l = System.currentTimeMillis();
            }

            @Override
            public String map(Tuple2<Boolean, Row> tuple2) throws Exception {
                if(tuple2.f0){
                    if(!"INSERT".equals(tuple2.f1.getKind().toString())){
                        return  "该数据库操作为更新操作，暂时不处理： "+tuple2.toString();
                    }else {
                   //row 转化成 bean
                    InputValue inputValue = new InputValue(
                            String.valueOf(tuple2.f1.getField(0)),
                            String.valueOf(tuple2.f1.getField(1)),
                            String.valueOf(tuple2.f1.getField(2)),
                            String.valueOf(tuple2.f1.getField(3)),
                            String.valueOf(tuple2.f1.getField(4)),
                            String.valueOf(tuple2.f1.getField(5)),
                            String.valueOf(tuple2.f1.getField(6)),
                            String.valueOf(tuple2.f1.getField(7)));
                    InputValue.objectToTrim(inputValue);
                    HashMap<String, String> param = new HashMap<>(16);
                    param.put("Content-Type","application/json;charset=UTF-8");
                        return http.sync("/rest/v1.0.1/database/insertRes")
                                .addHeader(param)
                                .bodyType("json")
                                //设置请求体json格式
                                .setBodyPara(JSON.toJSONString(inputValue))
                                .post()
                                .getBody()
                                .toString();
                    }
                }else {
                    return "该数据库操作为删除操作，暂时不处理： "+tuple2.toString();
                }
            }
        }).print();
        env.execute("cdc");
    }
}
