package cdc;

import bean.InputValue;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinaway.http.client.HttpClientResponse;
import com.chinaway.http.client.model.Response;
import com.ejlchina.okhttps.GsonMsgConvertor;
import com.ejlchina.okhttps.HTTP;
import org.apache.flink.api.common.functions.RichMapFunction;
import org.apache.flink.api.java.ExecutionEnvironment;
import org.apache.flink.api.java.io.CsvReader;
import org.apache.flink.api.java.operators.DataSource;
import org.apache.flink.configuration.Configuration;

import java.util.HashMap;

/**
 * @program: sunwin_metro
 * @description: 从csv读取故障列表输出到neo4j 构建neo4j图谱
 * @author: xuYJ
 * @create: 2021-06-23 09:20
 **/
public class NeoCsv {
    public static void main(String[] args) throws Exception {
        ExecutionEnvironment env = ExecutionEnvironment.getExecutionEnvironment();
        env.setParallelism(1);
        DataSource<InputValue> source = env.readCsvFile("C:\\Users\\daoda\\Desktop\\地铁项目\\新增ups关系.csv")
                .ignoreFirstLine()
                .pojoType(InputValue.class, "phenomenonName", "operationName", "reasonName", "reasonSolve", "locationName", "locationAbbreviation", "equipmentName", "equipmentAbbreviation");
        source.map(new RichMapFunction<InputValue, String>() {
            HTTP http;
            long l;

            @Override
            public void open(Configuration parameters) throws Exception {
                super.open(parameters);
                http = HTTP.builder()
                        .baseUrl("http://localhost:8080")
                        .addMsgConvertor(new GsonMsgConvertor())
                        .build();
                l = System.currentTimeMillis();
            }

            @Override
            public String map(InputValue inputValue) throws Exception {
                InputValue.objectToTrim(inputValue);
                HashMap<String, String> param = new HashMap<>(16);
                param.put("Content-Type", "application/json;charset=UTF-8");
                //同步
                return http.sync("/rest/v1.0.1/database/insertRes")
                        .addHeader(param)
                        .bodyType("json")
                        //设置请求体json格式
                        .setBodyPara(JSON.toJSONString(inputValue))
                        .post()
                        .getBody()
                        .toString();

            }

            @Override
            public void close() throws Exception {
                System.out.println("花费时间 = " + (System.currentTimeMillis() - l));
                super.close();
            }
        }).print();
    }
}
