package neo4j.controller;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.serializer.SerializerFeature;
import neo4j.model.test.Device;
import neo4j.model.test.MetroSystem;
import neo4j.relation.test.FeatureInfoGroupByDevice;
import neo4j.relation.test.FeatureSystem;
import neo4j.repository.test.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.*;

/**
 * @author xuYJ
 */
@RestController
@RequestMapping("/rest/v1.0.1/database")
public class MetroController {
    ExecutorService executor = Executors.newCachedThreadPool();
    @Autowired
    DeviceRepository deviceService;

    @Autowired
    SystemRepository systemService;

    @Autowired
    FeatureRepository featureService;

    @Autowired
    SystemDeviceRepository sysDevService;

    @Autowired
    DeviceFeatureRepository devFeaService;



    @RequestMapping("/getSysDevRes")
    public String getSysDevRes() {
        return JSON.toJSONString(sysDevService.findSystemDevices());
    }

    /**
     * 查询设备 -> 设备属性 关系
     */
    @RequestMapping("/getDevFeaRes")
    public String getDevFeaRes(@RequestParam(value = "deviceName",required = false) String deviceName) {
        if (null != deviceName && !"".equals(deviceName) && !deviceName.isEmpty()) {
            return JSONArray.toJSONString(devFeaService.findDeviceFeatures(deviceName), SerializerFeature.DisableCircularReferenceDetect);
        } else {
            return JSONArray.toJSONString(devFeaService.findDeviceFeatures(),SerializerFeature.DisableCircularReferenceDetect);
        }
    }

    /**
     * 查询系统名
     */
    @RequestMapping("/getSystemInfo")
    public String getSystemInfo(@RequestParam(value = "systemName") String systemName) throws ExecutionException, InterruptedException {
        long l = System.currentTimeMillis();
        FutureTask<Integer> task =null;
        task = new FutureTask<>(() -> {
            Thread.sleep(10000);
            return 10;
        });
        executor.execute(task);

        FutureTask<Integer> task2 =null;
        task2 = new FutureTask<>(() -> {
            Thread.sleep(10000);
            return 10;
        });
        executor.execute(task2);
        Integer integer = task.get();
        Integer integer1 = task2.get();

        return JSON.toJSONString(systemService.findByName(systemName));
    }

    /**
     * 查询设备名信息
     */
    @RequestMapping("/getDeviceInfo")
    public String getDeviceInfo(@RequestParam(value = "deviceName") String deviceName) {
        return JSON.toJSONString(deviceService.findByName(deviceName));
    }

    /**
     * 查询地铁系统名
     */
    @RequestMapping("/getDevices")
    public String getSystemDevice(@RequestParam(value = "systemName") String systemName) {
        List<Device> devices = deviceService.findDevices(systemName);
        return JSONArray.toJSONString(devices);
    }

    /**
     * 查询设备关系，根据设备属性
     */
    @RequestMapping("/getDeviceRes")
    public Map<String, List<Device>> getDevicesRes(@RequestParam(value = "featureName", required = false) String featureName) {
        List<FeatureInfoGroupByDevice> groupByFeature;
        HashMap<String, List<Device>> result = new HashMap<>(16);
        if (null != featureName && !"".equals(featureName) && !featureName.isEmpty()) {
            groupByFeature = deviceService.findFeatureInfoGroupByDeviceWithName(featureName);
            groupByFeature.forEach(featureInfoGroupByDevice -> result.put(featureInfoGroupByDevice.getFeatureName(), featureInfoGroupByDevice.getDevices()));
        } else {
            groupByFeature = deviceService.findFeatureInfoGroupByDevice();
            groupByFeature.forEach(featureInfoGroupByDevice -> result.put(featureInfoGroupByDevice.getFeatureName(), featureInfoGroupByDevice.getDevices()));
        }

        return result;
    }

    /**
     * 查询系统关系，根据设备属性
     */
    @RequestMapping("/getSystemRes")
    public Map<String, List<MetroSystem>> getSystemRes(@RequestParam(value = "featureName", required = false) String featureName) {
        List<FeatureSystem> groupByFeature;
        HashMap<String, List<MetroSystem>> result = new HashMap<>(16);
        if (null != featureName && !"".equals(featureName) && !featureName.isEmpty()) {
            groupByFeature = systemService.findFeatureBySys(featureName);
            groupByFeature.forEach(featureInfoGroupByDevice -> result.put(featureInfoGroupByDevice.getFeatureName(), featureInfoGroupByDevice.getMetroSystems()));
        } else {
            groupByFeature = systemService.findFeatureBySys();
            groupByFeature.forEach(featureInfoGroupByDevice -> result.put(featureInfoGroupByDevice.getFeatureName(), featureInfoGroupByDevice.getMetroSystems()));
        }
        return result;
    }


    /**
     * 根据设备名查询所有属性
     */
    @RequestMapping("/getFeatures")
    public String getFeatures(@RequestParam(value = "deviceName", required = false) String deviceName) {
        if (null != deviceName && !"".equals(deviceName) && !deviceName.isEmpty()) {
            return JSONArray.toJSONString(featureService.findFeature(deviceName));
        } else {
            return JSONArray.toJSONString(featureService.findFeature());
        }
    }

    /**
     * 根据属性名查询所有属性信息
     */
    @RequestMapping("/getFeatureInfo")
    public String getFeatureInfo(@RequestParam(value = "featureName") String featureName) {
        return JSONArray.toJSONString(featureService.findByName(featureName));

    }

}
